/** options for each client instance */
interface ClientOptions extends RequestInit {
    /** set the common root URL for all API requests */
    baseUrl?: string;
}
interface BaseParams {
    path?: Record<string, unknown>;
    query?: Record<string, unknown>;
}
type TruncatedResponse = Omit<Response, 'arrayBuffer' | 'blob' | 'body' | 'clone' | 'formData' | 'json' | 'text'>;
/** Infer request/response from content type */
type Unwrap<T> = T extends {
    content: {
        'application/json': {
            schema: any;
        };
    };
} ? T['content']['application/json']['schema'] : T extends {
    content: {
        'application/json': any;
    };
} ? T['content']['application/json'] : T extends {
    content: {
        '*/*': {
            schema: any;
        };
    };
} ? T['content']['*/*']['schema'] : T extends {
    content: {
        '*/*': any;
    };
} ? T['content']['*/*'] : T;
export default function createClient<T>(defaultOptions?: ClientOptions): {
    /** Call a GET endpoint */
    get<U extends keyof T, M extends keyof T[U]>(url: T[U] extends {
        get: any;
    } ? U : never, options: (T[U] extends {
        parameters: any;
    } ? {
        params: T[U]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U][M] extends {
        parameters: any;
    } ? {
        params: T[U][M]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U][M] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U][M]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]]["responses"] extends infer T_1 ? T_1 extends T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]]["responses"] ? T_1 extends {
            200: any;
        } ? T_1[200] : T_1 extends {
            201: any;
        } ? T_1[201] : T_1 extends {
            default: any;
        } ? T_1["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]]["responses"] extends infer T_2 ? T_2 extends T[T[U] extends {
            get: any;
        } ? U : never][keyof T[T[U] extends {
            get: any;
        } ? U : never]]["responses"] ? T_2 extends {
            500: any;
        } ? T_2[500] : T_2 extends {
            404: any;
        } ? T_2[404] : T_2 extends {
            default: any;
        } ? T_2["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a PUT endpoint */
    put<U_1 extends keyof T, M_1 extends keyof T[U_1]>(url: T[U_1] extends {
        put: any;
    } ? U_1 : never, options: (T[U_1] extends {
        parameters: any;
    } ? {
        params: T[U_1]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_1][M_1] extends {
        parameters: any;
    } ? {
        params: T[U_1][M_1]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_1][M_1] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_1][M_1]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]]["responses"] extends infer T_3 ? T_3 extends T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]]["responses"] ? T_3 extends {
            200: any;
        } ? T_3[200] : T_3 extends {
            201: any;
        } ? T_3[201] : T_3 extends {
            default: any;
        } ? T_3["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]]["responses"] extends infer T_4 ? T_4 extends T[T[U_1] extends {
            put: any;
        } ? U_1 : never][keyof T[T[U_1] extends {
            put: any;
        } ? U_1 : never]]["responses"] ? T_4 extends {
            500: any;
        } ? T_4[500] : T_4 extends {
            404: any;
        } ? T_4[404] : T_4 extends {
            default: any;
        } ? T_4["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a POST endpoint */
    post<U_2 extends keyof T, M_2 extends keyof T[U_2]>(url: T[U_2] extends {
        post: any;
    } ? U_2 : never, options: (T[U_2] extends {
        parameters: any;
    } ? {
        params: T[U_2]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_2][M_2] extends {
        parameters: any;
    } ? {
        params: T[U_2][M_2]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_2][M_2] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_2][M_2]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]]["responses"] extends infer T_5 ? T_5 extends T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]]["responses"] ? T_5 extends {
            200: any;
        } ? T_5[200] : T_5 extends {
            201: any;
        } ? T_5[201] : T_5 extends {
            default: any;
        } ? T_5["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]]["responses"] extends infer T_6 ? T_6 extends T[T[U_2] extends {
            post: any;
        } ? U_2 : never][keyof T[T[U_2] extends {
            post: any;
        } ? U_2 : never]]["responses"] ? T_6 extends {
            500: any;
        } ? T_6[500] : T_6 extends {
            404: any;
        } ? T_6[404] : T_6 extends {
            default: any;
        } ? T_6["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a DELETE endpoint */
    del<U_3 extends keyof T, M_3 extends keyof T[U_3]>(url: T[U_3] extends {
        delete: any;
    } ? U_3 : never, options: (T[U_3] extends {
        parameters: any;
    } ? {
        params: T[U_3]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_3][M_3] extends {
        parameters: any;
    } ? {
        params: T[U_3][M_3]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_3][M_3] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_3][M_3]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]]["responses"] extends infer T_7 ? T_7 extends T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]]["responses"] ? T_7 extends {
            200: any;
        } ? T_7[200] : T_7 extends {
            201: any;
        } ? T_7[201] : T_7 extends {
            default: any;
        } ? T_7["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]]["responses"] extends infer T_8 ? T_8 extends T[T[U_3] extends {
            delete: any;
        } ? U_3 : never][keyof T[T[U_3] extends {
            delete: any;
        } ? U_3 : never]]["responses"] ? T_8 extends {
            500: any;
        } ? T_8[500] : T_8 extends {
            404: any;
        } ? T_8[404] : T_8 extends {
            default: any;
        } ? T_8["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a OPTIONS endpoint */
    options<U_4 extends keyof T, M_4 extends keyof T[U_4]>(url: T[U_4] extends {
        options: any;
    } ? U_4 : never, options: (T[U_4] extends {
        parameters: any;
    } ? {
        params: T[U_4]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_4][M_4] extends {
        parameters: any;
    } ? {
        params: T[U_4][M_4]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_4][M_4] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_4][M_4]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]]["responses"] extends infer T_9 ? T_9 extends T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]]["responses"] ? T_9 extends {
            200: any;
        } ? T_9[200] : T_9 extends {
            201: any;
        } ? T_9[201] : T_9 extends {
            default: any;
        } ? T_9["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]]["responses"] extends infer T_10 ? T_10 extends T[T[U_4] extends {
            options: any;
        } ? U_4 : never][keyof T[T[U_4] extends {
            options: any;
        } ? U_4 : never]]["responses"] ? T_10 extends {
            500: any;
        } ? T_10[500] : T_10 extends {
            404: any;
        } ? T_10[404] : T_10 extends {
            default: any;
        } ? T_10["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a HEAD endpoint */
    head<U_5 extends keyof T, M_5 extends keyof T[U_5]>(url: T[U_5] extends {
        head: any;
    } ? U_5 : never, options: (T[U_5] extends {
        parameters: any;
    } ? {
        params: T[U_5]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_5][M_5] extends {
        parameters: any;
    } ? {
        params: T[U_5][M_5]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_5][M_5] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_5][M_5]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]]["responses"] extends infer T_11 ? T_11 extends T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]]["responses"] ? T_11 extends {
            200: any;
        } ? T_11[200] : T_11 extends {
            201: any;
        } ? T_11[201] : T_11 extends {
            default: any;
        } ? T_11["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]]["responses"] extends infer T_12 ? T_12 extends T[T[U_5] extends {
            head: any;
        } ? U_5 : never][keyof T[T[U_5] extends {
            head: any;
        } ? U_5 : never]]["responses"] ? T_12 extends {
            500: any;
        } ? T_12[500] : T_12 extends {
            404: any;
        } ? T_12[404] : T_12 extends {
            default: any;
        } ? T_12["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a PATCH endpoint */
    patch<U_6 extends keyof T, M_6 extends keyof T[U_6]>(url: T[U_6] extends {
        patch: any;
    } ? U_6 : never, options: (T[U_6] extends {
        parameters: any;
    } ? {
        params: T[U_6]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_6][M_6] extends {
        parameters: any;
    } ? {
        params: T[U_6][M_6]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_6][M_6] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_6][M_6]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]]["responses"] extends infer T_13 ? T_13 extends T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]]["responses"] ? T_13 extends {
            200: any;
        } ? T_13[200] : T_13 extends {
            201: any;
        } ? T_13[201] : T_13 extends {
            default: any;
        } ? T_13["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]]["responses"] extends infer T_14 ? T_14 extends T[T[U_6] extends {
            patch: any;
        } ? U_6 : never][keyof T[T[U_6] extends {
            patch: any;
        } ? U_6 : never]]["responses"] ? T_14 extends {
            500: any;
        } ? T_14[500] : T_14 extends {
            404: any;
        } ? T_14[404] : T_14 extends {
            default: any;
        } ? T_14["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
    /** Call a TRACE endpoint */
    trace<U_7 extends keyof T, M_7 extends keyof T[U_7]>(url: T[U_7] extends {
        trace: any;
    } ? U_7 : never, options: (T[U_7] extends {
        parameters: any;
    } ? {
        params: T[U_7]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_7][M_7] extends {
        parameters: any;
    } ? {
        params: T[U_7][M_7]["parameters"];
    } : {
        params?: BaseParams | undefined;
    }) & (T[U_7][M_7] extends {
        requestBody: any;
    } ? {
        body: Unwrap<T[U_7][M_7]["requestBody"]>;
    } : {
        body?: undefined;
    }) & Omit<RequestInit, "body">): Promise<{
        data: T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]]["responses"] extends infer T_15 ? T_15 extends T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]]["responses"] ? T_15 extends {
            200: any;
        } ? T_15[200] : T_15 extends {
            201: any;
        } ? T_15[201] : T_15 extends {
            default: any;
        } ? T_15["default"] : unknown : never : never> : unknown;
        error?: undefined;
        response: TruncatedResponse;
    } | {
        data?: undefined;
        error: T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]] extends {
            responses: any;
        } ? Unwrap<T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]]["responses"] extends infer T_16 ? T_16 extends T[T[U_7] extends {
            trace: any;
        } ? U_7 : never][keyof T[T[U_7] extends {
            trace: any;
        } ? U_7 : never]]["responses"] ? T_16 extends {
            500: any;
        } ? T_16[500] : T_16 extends {
            404: any;
        } ? T_16[404] : T_16 extends {
            default: any;
        } ? T_16["default"] : unknown : never : never> : unknown;
        response: TruncatedResponse;
    }>;
};
export {};
